/*
dttools is Copyright (C) 2002 Douglas Thain
This software is released under the GNU General Public License.
See the file COPYING for details.
*/


#include "safe_file.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <limits.h>
#include <unistd.h>

struct safe_file {
	char *name;
	char *tempname;
	FILE *stream;
};

static void safe_file_delete( struct safe_file *s )
{
	if(s) {
		if(s->name) free(s->name);
		if(s->tempname) free(s->tempname);
		free(s);
	}
}

struct safe_file * safe_file_begin( const char *name )
{
	char tempname[PATH_MAX];
	struct safe_file *s;

	s = malloc(sizeof(*s));
	if(!s) return 0;

	memset(s,0,sizeof(*s));

	s->name = strdup(name);
	if(!s->name) goto failure;

	sprintf(tempname,name,"%s.tmp",name);
	s->tempname = strdup(tempname);
	if(!s->tempname) goto failure;

	s->stream = fopen(tempname,"w");
	if(!s->stream) goto failure;

	return s;
	
	failure:
	safe_file_delete(s);
	return 0;
}

FILE * safe_file_stream( struct safe_file *s )
{
	return s->stream;
}

int safe_file_commit( struct safe_file *s )
{
	int result;

	result = fflush(s->stream);
	if(result!=0) goto failure;

	result = fsync(fileno(s->stream));
	if(result!=0) goto failure;

	result = fclose(s->stream);
	s->stream = 0;
	if(result!=0) goto failure;

	result = rename(s->tempname,s->name);
	if(result!=0) goto failure;
	
	safe_file_delete(s);
	return 1;

	failure:
	safe_file_delete(s);
	return 0;
}

void  safe_file_abort( struct safe_file *s )
{
	fclose(s->stream);
	unlink(s->tempname);
	safe_file_delete(s);
}
